
export elastix_webmodule_path="/var/www/html/modules"

elastix_adjust_khomp_config ()
{
	echo "Adjusting channel configuration for Elastix"

	if [ ! -f "${ROOT}/etc/asterisk/khomp.conf" ]
	then
		echo "Unable to find khomp.conf"
		exit 1
	fi

	sed -i 's/; context-fxs = khomp-DD-CC/context-fxs = from-internal/g' ${ROOT}/etc/asterisk/khomp.conf
	sed -i 's/; context-fxo = khomp-DD-CC/context-fxo = from-pstn/g' ${ROOT}/etc/asterisk/khomp.conf
	sed -i 's/; context-gsm-call = khomp-DD-CC/context-gsm-call = from-pstn/g' ${ROOT}/etc/asterisk/khomp.conf
	sed -i 's/; context-digital = khomp-DD-LL/context-digital = from-pstn/g' ${ROOT}/etc/asterisk/khomp.conf

    sed -i 's/; load-error = failure/load-error = skip/g' ${ROOT}/etc/asterisk/khomp.conf
}

elastix_install_webmodule ()
{
	echo "Installing Khomp web module for Elastix"

	make_dir_copy "${ROOT}/${elastix_webmodule_path}/khomp" "${PACKDIR}/webportal/" tree 755
	make_dir_copy "${ROOT}/${elastix_webmodule_path}/kconfig" "${PACKDIR}/webportal/KWebConfig" tree 755
	make_dir_copy "${ROOT}/${elastix_webmodule_path}/kmonitor" "${PACKDIR}/webportal/KWebMonitor" tree 755
	make_dir_copy "${ROOT}/${elastix_webmodule_path}/kdiagnostic" "${PACKDIR}/webportal/KWebLogViewer" tree 755

	rm -rf "${ROOT}/${elastix_webmodule_path}/khomp/KWebConfig"
	rm -rf "${ROOT}/${elastix_webmodule_path}/khomp/KWebMonitor"
	rm -rf "${ROOT}/${elastix_webmodule_path}/khomp/KWebLogViewer"

	mv -fv "${ROOT}/${elastix_webmodule_path}/kconfig/elastix-index.php" "${ROOT}/${elastix_webmodule_path}/kconfig/index.php"
	mv -fv "${ROOT}/${elastix_webmodule_path}/kmonitor/elastix-index.php" "${ROOT}/${elastix_webmodule_path}/kmonitor/index.php"
	mv -fv "${ROOT}/${elastix_webmodule_path}/kdiagnostic/elastix-index.php" "${ROOT}/${elastix_webmodule_path}/kdiagnostic/index.php"

	mv -fv "${ROOT}/${elastix_webmodule_path}/khomp/imagens/refreshing-elastix.gif" "${ROOT}/${elastix_webmodule_path}/khomp/imagens/refreshing.gif"

    touch "${ROOT}/opt/khomp/webserver/skip-kwebserver"
}

# Installing to a RPM buildroot
elastix_rpm_install ()
{
    ROOT=$1
    LIBDIR=$2

    mkdir -p ${ROOT}/usr/lib ${ROOT}/etc ${ROOT}${LIBDIR}/asterisk/modules
    export PATH=/usr/sbin:/sbin:$PATH

    export OPT_PURGE="no"
    export OPT_NO_INSTALL="no"
    export OPT_UNCOMPRESS_ONLY="no"
    export OPT_REPACKAGE="no"
    export OPT_VERBOSE="no"

    export OPT_BUILD="auto"
    export OPT_ASTERISK="auto"

    export OPT_SKIP_CHAN_KHOMP="no"

    export OPT_SKIP_KERNEL_MOD="no"
    export OPT_SKIP_WEBPORTAL="yes"
    export OPT_SKIP_WIZARD="yes"

    export PACKDIR=.

    if [ $LIBDIR == "/usr/lib64" ] ; then
        export arch_numbits=64
    else
        export arch_numbits=""
    fi

    msg ()
    {
	    echo "$@"
    }	

    export OPT_INSTALLING_RPM_ROOT="yes"

    source ./functions-generic.sh
    source ./instalador-k3l.sh && k3l_install
    source ./instalador-channel.sh && channel_install
}

elastix_fix_chkconfig_services ()
{
    sed -i '/#!\/bin\/sh/a \
\
# \
# k3lserver      Startup script for the KHOMP server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/k3lserver

    sed -i '/#!\/bin\/bash/a \
\
# \
# khompdrv     Startup script for the KHOMP drivers \
# \
# chkconfig: - 87 07 \
# description: KHOMP telephony card drivers \
\
' ${ROOT}/etc/init.d/khompdrv

    sed -i '/#!\/bin\/sh/a \
\
# \
# khomp-services      Startup script for the KHOMP services daemons \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/khomp-services


    sed -i '/#!\/bin\/sh/a \
\
# \
# kibs      Startup script for the KHOMP Integrated Boot Server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/kibs

    sed -i '/#!\/bin\/sh/a \
\
# \
# klogserver      Startup script for the KHOMP log server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/klogserver

    sed -i '/#!\/bin\/sh/a \
\
# \
# kqueryserver      Startup script for the KHOMP query server daemon \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/kqueryserver

    sed -i '/#!\/bin\/sh/a \
\
# \
# kwebserver      Startup script for the KHOMP webserver daemon  \
# \
# chkconfig: - 88 15 \
# description: The KHOMP server manages communication between Asterisk and KHOMP telephony cards \
\
' ${ROOT}/etc/init.d/kwebserver


}

elastix_adjust_kibs_config ()
{
	local OPT_FIRMWARE_DIR='/tftpboot'

    if grep -q "AlternativePath" ${ROOT}/etc/khomp/config/kibs.yaml 
    then
        sed -i 's,AlternativePath:.*,AlternativePath: /'$OPT_FIRMWARE_DIR',g' ${ROOT}/etc/khomp/config/kibs.yaml
    else
        sed -i 's,\(^[ ]*["]\?tftp["]\?:.*$\),\1\n        AlternativePath: /'$OPT_FIRMWARE_DIR',g' ${ROOT}/etc/khomp/config/kibs.yaml
    fi
}

elastix_support_install ()
{
	elastix_adjust_kibs_config
	elastix_adjust_khomp_config
	elastix_fix_chkconfig_services
	elastix_install_webmodule
}

# These are post install functions
elastix_add_services ()
{
	for i in kibs klogserver kqueryserver k3lserver khompdrv ; do
		/sbin/chkconfig --add $i
		/sbin/chkconfig $i on
	done

	/sbin/chkconfig --add khomp-services
	/sbin/chkconfig khomp-services off

	/sbin/chkconfig --add kwebserver
	/sbin/chkconfig kwebserver off
}

elastix_menumerge ()
{
	if type "elastix-menumerge" &> /dev/null && [ -e "${elastix_webmodule_path}/khomp/elastix-menu.xml" ];
	then
		elastix-menumerge "${elastix_webmodule_path}/khomp/elastix-menu.xml"
	else
	    echo "Unable to execute 'elastix-menumerge'"
		exit 1
	fi
}

elastix_services_start ()
{
	/sbin/service khomp-services start
}

elastix_set_permissions ()
{
	chown -R "asterisk:asterisk" /etc/khomp/config/*
	chown -R "asterisk:asterisk" /etc/khomp/config/.default.*

	chown -R "asterisk:asterisk" /var/www/html/modules/khomp
	chown -R "asterisk:asterisk" /var/www/html/modules/kmonitor
	chown -R "asterisk:asterisk" /var/www/html/modules/kconfig
	chown -R "asterisk:asterisk" /var/www/html/modules/kdiagnostic
}

elastix_set_sudoers ()
{
	grep khomp-sudo /etc/sudoers > /dev/null || echo "asterisk ALL = NOPASSWD: /usr/sbin/khomp-sudo" >> /etc/sudoers
}

# It should be independent of any variable built by the other installation scripts
elastix_post_install ()
{
	elastix_set_sudoers
	elastix_set_permissions
	elastix_add_services
	elastix_menumerge
}

# Uninstall stuff down here
elastix_menuremove ()
{
	if type "elastix-menuremove" &> /dev/null && [ -e "${elastix_webmodule_path}/khomp/elastix-menu.xml" ];
	then
		elastix-menuremove "khomp"
	else
		echo "Unable to find command 'elastix-menuremove'"
		exit 1
	fi
}

elastix_unset_sudoers ()
{
	sed -i '/asterisk[[:space:]]*ALL[[:space:]]*=[[:space:]]*NOPASSWD[[:space:]]*:[[:space:]]\/usr\/sbin\/khomp-sudo/d' /etc/sudoers
}

elastix_pre_uninstall ()
{
	elastix_unset_sudoers
	elastix_menuremove
}


