<?php
    require_once '../constants.php';
    require_once KROOT . '/classes/KSystem.php';
    $app = KSystem::getInstance( );
    $app->redirectIfNotLogged( );

    require_once KROOT . '/smarty-libs/Smarty.class.php';
    require_once KROOT . '/topo.php';
    require_once KROOT . '/rodape.php';
    require_once KROOT . '/util.php';
    require_once KCONFIG . '/languages/dictionary.php';
    require_once KCONFIG . '/classes/classKSistema.php';

    if( !isset( $_SESSION['sis'] ) )
    {
        $sistema = new KSistema( $app->getLanguage( ) );
        $_SESSION['sis'] = serialize( $sistema );
    }
    else
    {
        $sistema = unserialize( $_SESSION['sis'] );
        $sistema->setLanguage( $app->getLanguage( ) );
        $_SESSION['sis'] = serialize( $sistema );
    }

    $smarty = new Smarty( );
    $smarty->assign( 'FOOTER', Rodape::printRodape( ) );

    require_once KCONFIG . '/ajax/includeAjaxFiles.php';

    $ajaxTo = isset( $_REQUEST['ajaxTo'] ) ? $_REQUEST['ajaxTo'] : NULL;
    if( $ajaxTo )
    {
        // test if function exists, then call
        if( function_exists( $AjaxFunctions[$ajaxTo] ) )
        {
            echo call_user_func_array( $AjaxFunctions[$ajaxTo], array( $_REQUEST ) );
        }
        else
        {
            echo "Undefined function {$ajaxTo}";
        }
        exit ;
    }
    $smarty->assign( 'MODULO_PAI', '..' );
    $smarty->assign( 'MODULO', '.' );

    $js_conflict = '<script type="text/javascript" src="/js/jquery/jquery.min.js"></script>';
    $js_conflict .= '<script type="text/javascript" src="/js/jquery/jquery-ui-1.8.14.custom.min.js"></script>';
    $smarty->assign( 'JS_CONFLICT', $js_conflict );

    $location = isset( $_REQUEST['location'] ) ? $_REQUEST['location'] : NULL;
    if( $location )
    {
        switch( $location )
        {
            case 'admin':
                $smarty->assign( 'LEGEND', $app->translate( 'administration' ) );
                $topo = Topo::printTopo( 'administration', $_SESSION['loginAccess']['configuration'], $_SESSION['loginAccess']['monitoring'], $_SESSION['loginAccess']['diagnostic'], $_SESSION['loginAccess']['administration'] );
                $smarty->assign( 'TOP', $topo );
                $smarty->assign( 'MODULE', 'Administration' );

                if( $_SESSION['loginAccess']['administration'] == 'full' )
                    $smarty->assign( 'FORM', printAdminInterface( $_REQUEST ) );
                else
                    $smarty->assign( 'FORM', printSelfAdministration( $_REQUEST ) );

                $smarty->display( 'themes/default/default_no_menu.tpl' );
                break;
            case 'downloadLicense':
                $name = array_key_exists( 'name', $_GET ) ? $_GET['name'] : null;
                if( is_null( $name ) )
                {
                    header( 'HTTP/1.0 500 Internal Server Error: No name' );
                }

                require_once KROOT . '/classes/KLicense.php';

                try
                {
                    $manager = new KLicenseManager( );
                    $data = $manager->getLicenseRaw( $name );

                    $size = strlen( $data );
                    header( 'Content-Type: octect/stream' );
                    header( "Content-Disposition: attachment; filename=\"{$name}\"" );
                    header( "Content-Length: {$size}" );
                    echo $data;
                }
                catch( KLicenseManagerException $e )
                {
                    header( "HTTP/1.0 500 Internal Server Error: {$e->getMessage()}" );
                }
                catch( KLicense $e )
                {
                    header( "HTTP/1.0 404 Not Found: {$e->getMessage()}" );
                }
                break;
        }
    }
    else
    {
        $smarty->assign( 'LEGEND', $app->translate( 'sisConfig' ) );
        $topo = Topo::printTopo( 'config', $_SESSION['loginAccess']['configuration'], $_SESSION['loginAccess']['monitoring'], $_SESSION['loginAccess']['diagnostic'], $_SESSION['loginAccess']['administration'], '..' );
        $smarty->assign( 'TOP', $topo );
        $smarty->assign( 'CONFIG_SAVED', $app->translate( 'configurationSaved' ) );

        if( $app->hasPermission( 'configuration' ) )
        {
            $menu = call_user_func_array( $AjaxFunctions['showTreeMenu'], array( $_REQUEST ) );
            $smarty->assign( 'MENU', $menu );

            if( IS_GATEWAY )
            {
                $smarty->assign( 'CHANGEVIEW', 'showRoutesGW' );
                $smarty->assign( 'SECTION', 'menu-routing-gw' );
            }
            else
            {
                $smarty->assign( 'CHANGEVIEW', 'showDevices' );
                $smarty->assign( 'SECTION', 'menu-device' );
            }
            $smarty->assign( 'FORM', showDevices( $_REQUEST ) );
        }
        else
        {
            $smarty->assign( 'MENU', '' );
            $smarty->assign( 'CHANGEVIEW', '' );
            $smarty->assign( 'SECTION', '' );
            $smarty->assign( 'FORM', $app->translate( 'pDenied' ) );
        }

        $smarty->display( 'themes/default/default.tpl' );
    }
