<?php
    require_once '../constants.php';
    require_once KROOT . '/classes/KSystem.php';
    $app = KSystem::getInstance( );
    $app->redirectIfNotLogged( );

    require_once KROOT . '/smarty-libs/Smarty.class.php';
    require_once KROOT . '/topo.php';
    require_once KROOT . '/rodape.php';
    require_once KROOT . '/util.php';
    require_once KCONFIG . '/classes/classKSistema.php';
    require_once KMONITOR . '/mapDevices.php';
    require_once KMONITOR . '/monitorService.php';
    require_once KMONITOR . '/monitorDevice.php';
    require_once KMONITOR . '/monitorLink.php';
    require_once KMONITOR . '/monitorChannel.php';
    require_once KMONITOR . '/languages/dictionary.php';

    global $sistema;

    if( !isset( $_SESSION['sis'] ) )
    {
        $sistema = new KSistema( $app->getLanguage( ) );
        $_SESSION['sis'] = serialize( $sistema );
    }
    else
    {
        $sistema = unserialize( $_SESSION['sis'] );
        $sistema->setLanguage( $app->getLanguage( ) );
    }

    $smarty = new Smarty( );

    if( $app->hasPermission( 'monitoring', array( 'read', 'full' ) ) )
    {
        $location = isset( $_REQUEST['location'] ) ? $_REQUEST['location'] : NULL;
        if( isset( $location ) )
        {
            switch($location)
            {
                case 'services':
                    $monitor = new MonitorService( );
                    $smarty->assign( "CONTENT", $monitor->showServices( ) );
                    $smarty->assign( "LEGEND", $app->translate( 'services' ) );
                    $smarty->assign( "MODULE", 'Services' );
                    $smarty->assign( "JSmodule", 'Services' );
                    break;

                case 'devices':
                    $monitor = new MonitorDevice( );
                    $smarty->assign( "CONTENT", $monitor->showDevices( ) );
                    $smarty->assign( "LEGEND", $app->translate( 'devices' ) );
                    $smarty->assign( "MODULE", 'Devices' );
                    $smarty->assign( "JSmodule", 'Devices' );
                    break;

                case 'links':
                    $monitor = new MonitorLink( );
                    $smarty->assign( "CONTENT", $monitor->showLinks( ) );
                    $smarty->assign( "LEGEND", $app->translate( 'links' ) );
                    $smarty->assign( "MODULE", 'Links' );
                    $smarty->assign( "JSmodule", 'Links' );
                    break;

                case 'channels':
                    $monitor = new MonitorChannel( );
                    $content = $monitor->getMessages( );
                    $content .= $monitor->getDevices( );
                    $smarty->assign( "CONTENT", $content );
                    $smarty->assign( "LEGEND", $app->translate( 'channels' ) );
                    $smarty->assign( "MODULE", 'Channels' );
                    $smarty->assign( "JSmodule", 'Channels' );
                    break;
                default:
                    $monitor = new MonitorDevice( );
                    $smarty->assign( "CONTENT", $monitor->showDevices( ) );
                    $smarty->assign( "LEGEND", $app->translate( 'devices' ) );
                    $smarty->assign( "MODULE", 'Devices' );
                    $smarty->assign( "JSmodule", 'Devices' );
                    break;
            }
        }

        $ajaxTo = isset( $_REQUEST['ajaxTo'] ) ? $_REQUEST['ajaxTo'] : NULL;
        if( isset( $ajaxTo ) )
        {
            require_once KROOT . '/json_util.php';
            require_once KMONITOR . '/ajax/includeAjaxFiles.php';
            if( function_exists( $AjaxFunctions[$ajaxTo] ) )
            {
                echo call_user_func_array( $AjaxFunctions[$ajaxTo], array( $_REQUEST ) );
            }
            else
            {
                echo "Undefined function {$ajaxTo}";
            }
            exit ;
        }
    }
    else
        $smarty->assign( 'CONTENT', $app->translate( 'noPermissionToThisPage' ) );

    $smarty->assign( 'MODULO_PAI', '..' );
    $smarty->assign( 'MODULO', '.' );

    $js_conflict = '<script type="text/javascript" src="../js/jquery/jquery.js"></script>';
    $js_conflict .= '<script type="text/javascript" src="../js/jquery/jquery.min.js"></script>';
    $smarty->assign( 'JS_CONFLICT', $js_conflict );

    $topo = Topo::printTopo( 'monitoring', $_SESSION['loginAccess']['configuration'], $_SESSION['loginAccess']['monitoring'], $_SESSION['loginAccess']['diagnostic'], $_SESSION['loginAccess']['administration'], '..' );
    $smarty->assign( "TOP", $topo );

    $rodape = Rodape::printRodape( );
    $smarty->assign( "FOOTER", $rodape );

    $SO = PHP_OS == "WINNT" ? "WIN" : "LINUX";
    $smarty->assign( "SO", $SO );

    $smarty->assign( "DESVIO_X", '10' );
    $smarty->assign( "DESVIO_Y", '500' );

    if( !IS_GATEWAY )
    {
        $dev = "<input id='devices-button' value='{$app->translate('devices')}' onclick=\"window.location = 'index.php?location=devices';colorMe(this.id);\" type='button' />";
        $smarty->assign( "DEVICES", $dev );
    }

    $smarty->assign( "LINKS", $app->translate( 'links' ) );
    $smarty->assign( "CHANNELS", $app->translate( 'channels' ) );
    $smarty->assign( "SERVICES", $app->translate( 'services' ) );
    $smarty->assign( "REFRESH", $app->translate( 'refresh' ) );
    $smarty->assign( "REFRESH_DELAY", $app->translate( 'refreshDelay' ) );
    $smarty->assign( "NOT_REFRESH", $app->translate( 'notRefresh' ) );
    $smarty->assign( "DELAY1", $app->translate( 'delay1' ) );
    $smarty->assign( "DELAY2", $app->translate( 'delay2' ) );
    $smarty->assign( "DELAY3", $app->translate( 'delay3' ) );

    $smarty->display( 'themes/default/default.tpl' );
