<?php
    /* include Querist */
    require_once ("querist.php");

    /* Important devices */
    define( "POPULATOR_NOT_ARRAY", "Not a array" );
    define( "POPULATOR_INCOMPATIBLE_DEVICE", "Device is not compatible with especifications" );
    /*
     * Responsible for checking the array semantics
     */
    class Checker
    {
        /*
         * Check the compatibity for the device,
         * with the array var analysis
         */
        static function checkDevice( &$device )
        {
            if( !isset( $device['model'] ) )
                return false;
            if( !isset( $device['board'] ) )
                return false;
            if( !isset( $device['serial'] ) )
                return false;
            if( !isset( $device['status'] ) )
                return false;
            if( !isset( $device['links'] ) )
                return false;
            if( !is_array( $device['links'] ) )
                return false;
            return true;
        }

    }

    /*
     * This class is responsible for populate all
     * device informations
     */
    class Populator
    {
        /* Last error ocurred will be setted here */
        static public $last_error = "";
        /* Basic check before all procedures */
        private static function basicCheck( &$device )
        {
            if( !is_array( $device ) )
            {
                Populator::$last_error = POPULATOR_NOT_ARRAY;
                return false;
            }
            if( !Checker::checkDevice( $device ) )
            {
                Populator::$last_error = POPULATOR_INCOMPATIBLE_DEVICE;
                return false;
            }
            return true;
        }

        /* Populate the linked device */
        public static function populateDevice( $device )
        {
            if( !Populator::basicCheck( $device ) )
                return true;
            $board_num = $device['board'];
            $query_serial = sprintf( 'QUERY k3l.Config.Device.%s.SerialNumber', $board_num );
            Querist::ask( $query_serial, $device['serial'] );

            $query_type = sprintf( 'QUERY k3l.DeviceType.%s', $device['serial'] );
            Querist::ask( $query_type, $device['type'] );

            if( $device['type'] != 22 )
                $device['model'] = Populator::getDeviceModel( $device['type'], $device['serial'] );
            else
            {
                $query_model_mod = sprintf( 'QUERY k3l.Config.EBS.%s.String', $device['serial'] );
                Querist::ask( $query_model_mod, $modelo );
                $query_model = sprintf( "QUERY k3l.Config.Device.%s.DeviceModel", $device['serial'] );
                Querist::ask( $query_model, $model );
                $device['model'] = "EBS-Modular " . Populator::getEBSDeviceModel( $model ) . " " . substr( $modelo, 0, -1 );
            }

            switch($device['type'])
            {
                case 14:
                case 15:
                    $device['status'] = 'N/A';
                    break;
                case 18:
                case 19:
                case 20:
                case 21:
                case 24:
                // EBS Modular
                case 22:
                case 23:
                    $query_status_ebs = sprintf( 'QUERY k3l.Status.Connected.%s', $device['serial'] );
                    Querist::ask( $query_status_ebs, $status_ebs );
                    if( $status_ebs == "" )
                        $device['status'] = "N/A";
                    else if( $status_ebs == 0 )
                        $device['status'] = "Down";
                    else
                        $device['status'] = "Up";
                    break;

                // case 'VoIP' :
                case 17:
                    if( MonitorService::checkService( "kmpserver" ) )
                        $device['status'] = 'Up';
                    else
                        $device['status'] = 'Down';
                    break;

                //If model is a board, the status is always UP
                default:
                    $device['status'] = 'Up';
                    break;
            }
            /*
             * Only E1, FXS, EBS_E1 and EBS_Mod have links
             */

            if( $device['type'] == 0 || $device['type'] == 7 || $device['type'] == 9 || $device['type'] == 10 || $device['type'] == 18 || $device['type'] == 19 || $device['type'] == 20 || $device['type'] == 21 || $device['type'] == 22 || $device['type'] == 24 )
            {
                $links_count = 0;
                $query_link_count = sprintf( "QUERY k3l.Config.Device.%s.LinkCount", $device['serial'] );
                Querist::ask( $query_link_count, $links_count );

                $tp = 'E1';
                if( $device['type'] == 24 )
                {
                    // EBSE1-HI
                    $links_count = $links_count * 2;
                    $tp = "HI";
                }

                $tmp = "";
                $tmp_arr = array( );
                for( $i = 0; $i < $links_count; $i++ )
                {

                    if( $device['status'] == "Up" )
                    {
                        $query_link_status = sprintf( "QUERY k3l.Status.Link.%s.%s.0." . $tp, $device['serial'], $i );
                        Querist::ask( $query_link_status, $tmp );

                        if( strlen( $tmp ) > 3 )
                            $tmp_arr[$i]['status'] = "-";
                        else
                        {
                            if( $tmp != 0 )
                                $tmp_arr[$i]['status'] = "Down";
                            else
                                $tmp_arr[$i]['status'] = "Up";
                        }
                    }
                    else
                        $tmp_arr[$i]['status'] = "N/A";
                }
                $device['links'] = $tmp_arr;
            }
            $fail_total = 0;
            $idle_total = 0;
            $call_total = 0;

            $query_fail_total = sprintf( 'QUERY k3l.Status.Channel.%s.0.TotalFail', $device['serial'] );
            Querist::ask( $query_fail_total, $fail_total );

            $query_idle_total = sprintf( 'QUERY k3l.Status.Channel.%s.0.TotalIdle', $device['serial'] );
            Querist::ask( $query_idle_total, $idle_total );

            $query_call_total = sprintf( 'QUERY k3l.Status.Channel.%s.0.TotalCall', $device['serial'] );
            Querist::ask( $query_call_total, $call_total );

            $device['channels'] = array( 'fail' => $fail_total, 'idle' => $idle_total, 'call' => $call_total );

            return $device;
        }

        public static function populateLink( $device )
        {

            $board_num = $device['board'];

            $query_serial = sprintf( 'QUERY k3l.Config.Device.%s.SerialNumber', $board_num );
            Querist::ask( $query_serial, $device['serial'] );

            $query_type = sprintf( 'QUERY k3l.DeviceType.%s', $device['serial'] );
            Querist::ask( $query_type, $device['type'] );

            switch($device['type'])
            {
                case 14:
                case 15:
                    $device['status'] = 'N/A';
                    break;
                case 18:
                case 19:
                case 20:
                case 24:
                case 21:
                    $query_status_ebs = sprintf( 'QUERY k3l.Status.Connected.%s', $device['serial'] );
                    Querist::ask( $query_status_ebs, $status_ebs );
                    if( $status_ebs == "" )
                        $device['status'] = "N/A";
                    else if( $status_ebs == 0 )
                        $device['status'] = "Down";
                    else
                        $device['status'] = "Up";
                    break;

                // case 'VoIP' :
                case 17:
                    if( MonitorService::checkService( "kmpserver" ) )
                        $device['status'] = 'Up';
                    else
                        $device['status'] = 'Down';
                default:
                    $device['status'] = 'Up';
                    break;
            }

            if( $device['type'] != 22 )
                $device['model'] = Populator::getDeviceModel( $device['type'], $device['serial'] );
            else
            {
                $query_model_mod = sprintf( 'QUERY k3l.Config.EBS.%s.String', $device['serial'] );
                Querist::ask( $query_model_mod, $modelo );
                $query_model = sprintf( "QUERY k3l.Config.Device.%s.DeviceModel", $device['serial'] );
                Querist::ask( $query_model, $model );
                $device['model'] = "EBS-Modular " . Populator::getEBSDeviceModel( $model ) . " " . substr( $modelo, 0, -1 );
            }

            $links_count = 0;
            $query_link_count = sprintf( "QUERY k3l.Config.Device.%s.LinkCount", $device['serial'] );
            Querist::ask( $query_link_count, $links_count );

            $tp = "E1";
            $tp2 = '';
            if( $device['type'] == 24 )
            {
                $links_count = $links_count * 2;
                $tp = "HI";
                $tp2 = "HI";
            }

            for( $i = 0; $i < $links_count; $i++ )
            {
                $device['links'][$i]['id'] = $i;

                $query_signaling = sprintf( "QUERY k3l.Config.Link$tp2.%s.%s.Signaling", $device['serial'], $i );
                Querist::ask( $query_signaling, $signaling );
                $device['links'][$i]['signaling'] = Populator::getSignaling( $signaling );

                $query_nick = sprintf( "QUERY k3l.Config.Link$tp2.%s.%s.Name", $device['serial'], $i );
                Querist::ask( $query_nick, $nick );
                $device['links'][$i]['nick'] = utf8_encode( $nick );

                $query_status = sprintf( 'QUERY k3l.Status.Link.%s.%s.0.' . $tp, $device['serial'], $i );
                Querist::ask( $query_status, $link_status );

                if( $device['status'] == "Up" )
                {
                    if( $link_status == 0 )
                    {
                        // link OK
                        $device['links'][$i]['status'] = "Up";
                        $device['links'][$i]['alarms']['SignalLost'] = 0;
                        $device['links'][$i]['alarms']['NetworkAlarm'] = 0;
                        $device['links'][$i]['alarms']['FrameSyncLost'] = 0;
                        $device['links'][$i]['alarms']['MultiframeSyncLost'] = 0;
                        $device['links'][$i]['alarms']['RemoteAlarm'] = 0;
                        $device['links'][$i]['alarms']['HighErrorRate'] = 0;
                        $device['links'][$i]['alarms']['UnknownAlarm'] = 0;
                        $device['links'][$i]['alarms']['E1Error'] = 0;
                        $device['links'][$i]['alarms']['NotInitialized'] = 0;

                        $query_variations = sprintf( 'QUERY k3l.Status.LinkErrorCounter.%s.%s', $device['serial'], $i );
                        Querist::ask( $query_variations, $resultVar );

                    }
                    else
                    {
                        $device['links'][$i]['status'] = "Down";
                        $variations = Populator::getVariations( $link_status );
                        $device['links'][$i]['alarms']['SignalLost'] = $variations[0];
                        $device['links'][$i]['alarms']['NetworkAlarm'] = $variations[1];
                        $device['links'][$i]['alarms']['FrameSyncLost'] = $variations[2];
                        $device['links'][$i]['alarms']['MultiframeSyncLost'] = $variations[3];
                        $device['links'][$i]['alarms']['RemoteAlarm'] = $variations[4];
                        $device['links'][$i]['alarms']['HighErrorRate'] = $variations[5];
                        $device['links'][$i]['alarms']['UnknownAlarm'] = $variations[6];
                        $device['links'][$i]['alarms']['E1Error'] = $variations[7];
                        $device['links'][$i]['alarms']['NotInitialized'] = $variations[8];

                        $query_variations = sprintf( 'QUERY k3l.Status.LinkErrorCounter.%s.%s', $device['serial'], $i );
                        Querist::ask( $query_variations, $resultVar );

                    }
                }
                else
                {
                    $device['links'][$i]['status'] = "Down";
                    $resultVar = "N/A";
                }

                if( $resultVar == "N/A" )

                    $variations = array( 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A', 'N/A' );
                else if( $resultVar != "" )
                    $variations = explode( ',', $resultVar );
                else
                    $variations = array( '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-', '-' );

                $device['links'][$i]['variations']['BlockedCount'] = $variations[0];
                $device['links'][$i]['variations']['SignalLostError'] = $variations[1];
                $device['links'][$i]['variations']['AlarmReceived'] = $variations[2];
                $device['links'][$i]['variations']['FrameSync'] = $variations[3];
                $device['links'][$i]['variations']['MultiFrameSync'] = $variations[4];
                $device['links'][$i]['variations']['RemoteAlarm'] = $variations[5];
                $device['links'][$i]['variations']['SlipAlarm'] = $variations[6];
                $device['links'][$i]['variations']['PRBS'] = $variations[7];
                $device['links'][$i]['variations']['WrongEBits'] = $variations[8];
                $device['links'][$i]['variations']['FIFOJitter'] = $variations[9];
                $device['links'][$i]['variations']['LostSyncTime'] = $variations[10];
                $device['links'][$i]['variations']['FrameAlignment'] = $variations[11];
                $device['links'][$i]['variations']['WrongFramesCount'] = $variations[12];
                $device['links'][$i]['variations']['BipolarViolation'] = $variations[13];
                $device['links'][$i]['variations']['CRC-4Error'] = $variations[14];
            }

            return $device;

        }

        public static function prePopulateChannels( $device )
        {
            $board_num = $device['board'];

            $query_serial = sprintf( 'QUERY k3l.Config.Device.%s.SerialNumber', $board_num );
            Querist::ask( $query_serial, $device['serial'] );

            $query_type = sprintf( 'QUERY k3l.DeviceType.%s', $device['serial'] );
            Querist::ask( $query_type, $device['type'] );

            if( $device['type'] != 22 )
                $device['model'] = Populator::getDeviceModel( $device['type'], $device['serial'] );
            else
            {
                $query_model_mod = sprintf( 'QUERY k3l.Config.EBS.%s.String', $device['serial'] );
                Querist::ask( $query_model_mod, $modelo );
                $query_model = sprintf( "QUERY k3l.Config.Device.%s.DeviceModel", $device['serial'] );
                Querist::ask( $query_model, $model );
                $device['model'] = "EBS-Modular ";
            }

            $links_count = 0;
            $query_link_count = sprintf( "QUERY k3l.Config.Device.%s.LinkCount", $device['serial'] );

            Querist::ask( $query_link_count, $links_count );

            if( $links_count == 0 )
                $device['links'][99] = array( 'channels' => array( 0 => 0 ) );
            else
            {
                for( $i = 0; $i < $links_count; $i++ )
                    $device['links'][$i] = array( 'channels' => array( 0 => 0 ) );
            }

            return $device;
        }

        public static function populateChannels( $device, $link, $firstCh = null, $lastCh = null )
        {
            // Device without links, just channels
            // if (isset($device['links'][99]))
            if( $link == 99 )
                return Populator::populateChannelsWithoutLink( $device, $firstCh, $lastCh );
            //Device with links
            else
                return Populator::populateChannelsFromLink( $device, $link );
        }

        public static function populateChannelsFromLink( $device, $link )
        {
            $board_num = $device['board'];

            $query_signaling = sprintf( "QUERY k3l.Config.Link.%s.%s.Signaling", $device['serial'], $link );
            Querist::ask( $query_signaling, $signaling );
            $signaling = Populator::getSignaling( $signaling );

            $query_voip = sprintf( 'QUERY k3l.Config.Device.%s.VoIPChannelCount', $device['serial'] );
            Querist::ask( $query_voip, $voipCount );

            $query_ch = sprintf( 'QUERY k3l.Config.Device.%s.ChannelCount', $device['serial'] );
            Querist::ask( $query_ch, $chCount );

            $voipMin = $chCount - $voipCount;

            // One link always have 30 channels
            for( $i = 0; $i < 30; $i++ )
            {
                $ch = (30 * $link) + $i;

                $query_status_channel = sprintf( 'QUERY k3l.Status.Channel.%s.%s.CallStatus', $device['serial'], $ch );
                Querist::ask( $query_status_channel, $status_channel );
                $status_tmp = Populator::getCallStatus( $status_channel );

                $query_addInfo = sprintf( 'QUERY k3l.Status.Channel.%s.%s.AddInfo', $device['serial'], $ch );
                Querist::ask( $query_addInfo, $addInfo );

                if( $device['type'] == 22 )
                    $addInfo = Populator::getAddInfoModular( $addInfo, $signaling );
                else
                    $addInfo = Populator::getAddInfo( $addInfo, $device['type'], $ch, $device['serial'], $voipMin );

                $query_info_channel = sprintf( 'QUERY k3l.ChannelStats.%s.%s', $device['serial'], $ch );
                Querist::ask( $query_info_channel, $info_channel );
                $infos = explode( ',', $info_channel );

                $inbound = isset( $infos[0] ) ? $infos[0] : '';
                $outbound = isset( $infos[1] ) ? $infos[1] : '';
                $succeed = isset( $infos[2] ) ? $infos[2] : '';
                $failed = isset( $infos[3] ) ? $infos[3] : '';

                $device['links'][$link]['channels'][$i] = array( 'callStatus' => $status_tmp, 'signaling' => $signaling, 'details' => $addInfo, 'inbound' => $inbound, 'outbound' => $outbound, 'succeed' => $succeed, 'failed' => $failed );
            }

            return $device;

        }

        public static function populateChannelsWithoutLink( $device, $firstCh, $lastCh )
        {

            $board_num = $device['board'];

            $query_voip = sprintf( 'QUERY k3l.Config.Device.%s.VoIPChannelCount', $device['serial'] );
            Querist::ask( $query_voip, $voipCount );
            $query_ch = sprintf( 'QUERY k3l.Config.Device.%s.ChannelCount', $device['serial'] );
            Querist::ask( $query_ch, $chCount );
            $voipMin = $chCount - $voipCount;

            if( isset( $firstCh ) && isset( $lastCh ) )
            {
                $i = $firstCh;
                $count_channel = $lastCh + 1;
            }
            else
            {
                $query_channel_count = sprintf( 'QUERY k3l.Config.Device.%s.ChannelCount', $device['serial'] );
                Querist::ask( $query_channel_count, $count_channel );
                $i = 0;
            }
            $query_gsm_channel_count = sprintf( 'QUERY k3l.Config.EBS.%s.GSMChannels', $device['serial'] );
            Querist::ask( $query_gsm_channel_count, $countGSM );

            $GSMch = explode( ',', $countGSM );

            for( $i; $i < $count_channel; $i++ )
            {
                $query_status_channel = sprintf( 'QUERY k3l.Status.Channel.%s.%s.CallStatus', $device['serial'], $i );
                Querist::ask( $query_status_channel, $status_channel );

                $status_tmp = Populator::getCallStatus( $status_channel );

                $query_signaling = sprintf( 'QUERY k3l.Config.Channel.%s.%s.Signaling', $device['serial'], $i );
                Querist::ask( $query_signaling, $sig );
                $signaling = Populator::getSignaling( $sig );

                $query_addInfo = sprintf( 'QUERY k3l.Status.Channel.%s.%s.AddInfo', $device['serial'], $i );
                Querist::ask( $query_addInfo, $addInfo );

                if( $device['type'] == 22 )
                    $addInfo = Populator::getAddInfoModular( $addInfo, $sig );
                else
                    $addInfo = Populator::getAddInfo( $addInfo, $device['type'], $i, $device['serial'], $voipMin );

                $query_inbound_channel = sprintf( 'QUERY k3l.ChannelStats.%s.%s', $device['serial'], $i );
                Querist::ask( $query_inbound_channel, $info_channel );
                $infos = explode( ',', $info_channel );

                $oper = "-";
                $strength = "-";

                if( in_array( $i, $GSMch ) )
                {
                    $query_strength = sprintf( 'QUERY k3l.Status.GSMChannel.%s.%s.SignalStrength.0', $device['serial'], $i );
                    Querist::ask( $query_strength, $strength );
                    if( $strength == '255' || $strength == '' )
                        $strength = "-";
                    else
                        $strength .= "%";

                    $query_oper = sprintf( 'QUERY k3l.Status.GSMChannel.%s.%s.OperName.0', $device['serial'], $i );
                    Querist::ask( $query_oper, $oper );
                    if( $oper == "" )
                        $oper = "-";
                }
                //link 99 means this device dont have links but have channels.
                $device['links'][99]['channels'][$i] = array( 'callStatus' => $status_tmp, 'signaling' => $signaling, 'details' => $addInfo, 'inbound' => $infos[0], 'outbound' => $infos[1], 'succeed' => $infos[2], 'failed' => $infos[3], 'oper' => utf8_encode( $oper ), 'strength' => $strength );
            }

            return $device;

        }

        public static function getVariations( $status )
        {
            $retorno = array( );
            // masks sequence: SignalLost, NetworkAlarm, FrameSyncLost,
            // MultiframeSyncLost, RemoteAlarm, HighErrorRate, UnknownAlarm,
            // E1Error,NotInitialized
            $mascaras = array( "00000001", "00000010", "00000100", "00001000", "00010000", "00100000", "01000000", "10000000", "11111111" );
            $status = decbin( $status );
            $status = Populator::zeroFill( $status, 8 );

            for( $i = 0; $i < count( $mascaras ); $i++ )
            {
                if( ($mascaras[$i] == $status) == $mascaras[$i] )
                    $retorno[] = 1;
                else
                    $retorno[] = 0;
            }
            return $retorno;
        }

        public static function getSignaling( $signal )
        {
            $signaling = array( 0 => "Inactive", 1 => "R2Digital", 2 => "ContinuosEM", 3 => "PulsedEM", 4 => "UserR2Digital", 5 => "Analog", 6 => "OpenCAS", 7 => "OpenR2", 8 => "SIP", 9 => "OpenCCS", 10 => "PRI EndPoint", 11 => "Analog Terminal", 12 => "PRI Network", 13 => "PRI Passive", 14 => "Line Side", 15 => "CAS EL7", 16 => "GSM", 17 => "E1LC", 18 => "ISUP", 19 => "ISUP Passive" );
            return $signaling[$signal];
        }

        public static function getCallStatus( $stats )
        {
            switch ($stats)
            {
                case '0':
                    //return "Free";
                    return "Idle";
                case '1':
                    //return "Incoming";
                    return "On Call";
                case '2':
                    // return "Outgoing";
                    return "On Call";
                default:
                    return "Fail";
            }
        }

        public static function getAddInfoEBS( $type, $addInfo )
        {
            switch ($type)
            {
                // case 'EBS E1' :
                case 18:
                // case 'EBS HI' :
                case 24:
                    return Populator::getAddInfoE1( $addInfo );
                // case 'EBS GSM' :
                case 21:
                    return Populator::getAddInfoGSM( $addInfo );
                // case 'EBS FXS' :
                case 20:
                    return Populator::getAddInfoFXS( $addInfo );
                // case 'EBS FXO' :
                case 19:
                    return Populator::getAddInfoFXO( $addInfo );
                default:
                    //EBS_Mod not implemented yet
                    //return Populator::getAddInfoMod($addInfo);
                    return "-";
            }
        }

        public static function getAddInfoModular( $addinfo, $signaling )
        {
            switch($signaling)
            {

                case 5:
                    return Populator::getAddInfoFXO( $addinfo );
                case 8:
                    return Populator::getAddInfoVoIP( $addinfo );
                case 11:
                    return Populator::getAddInfoFXS( $addinfo );
                case 16:
                    return Populator::getAddInfoGSM( $addinfo );
                default:
                    return Populator::getAddInfoE1( $addinfo );
            }
        }

        public static function getAddInfo( $addInfo, $type, $ch, $board, $voipMin )
        {
            if( $type != 17 )
            {
                switch ($type)
                {
                    case 18:
                    case 19:
                    case 20:
                    case 21:
                    case 24:
                        return Populator::getAddInfoEBS( $type, $addInfo );
                    case 11:
                    case 12:
                    case 14:
                    case 15:
                        return Populator::getAddInfoGSM( $addInfo );
                    case 1:
                    case 5:
                        return Populator::getAddInfoFXO( $addInfo );
                    case 9:
                    case 10:
                        return Populator::getAddInfoFXS( $addInfo );
                    case 0:
                    case 4:
                    case 6:
                    case 7:
                    case 13:
                        return Populator::getAddInfoE1( $addInfo );
                    default:
                        return "-";
                }
            }
            else
                return Populator::getAddInfoVoIP( $addInfo );
        }

        public static function getAddInfoFXO( $stats )
        {
            switch ($stats)
            {
                case '0':
                    return "Disable";
                case '1':
                    return "Enable";
                default:
                    return "Fail";
            }
        }

        public static function getAddInfoVoIP( $stats )
        {
            $retorno = array( );
            $mascaras = array( Populator::zeroFill( decbin( 16 ), 8 ), Populator::zeroFill( decbin( 64 ), 8 ) );
            $stats = decbin( $stats );
            $stats = Populator::zeroFill( $stats, 8 );
            if( $stats == 0 )
                return "Free";
            else
            {
                if( ($mascaras[0] & $stats) == $mascaras[0] )
                    $retorno[] = "OutgoingLock";

                if( ($mascaras[1] & $stats) == $mascaras[1] )
                    $retorno[] = "IncomingLock";

                return implode( ",", $retorno );
            }
        }

        public static function getAddInfoFXS( $status )
        {
            switch ($status)
            {
                case '0':
                    return "On Hook";
                case '1':
                    return "Off Hook";
                case '2':
                    return "Ringing";
                default:
                    return "Fail";
            }
        }

        public static function getAddInfoGSM( $status )
        {
            switch ($status)
            {
                case '0':
                    return "Idle";
                case '1':
                    return "Call In Progress";
                case '2':
                    return "SMSInProgress";
                case '3':
                    return "ModemError";
                case '4':
                    return "SIMCardError";
                case '5':
                    return "NetworkError";
                default:
                    return "Not Ready";
            }
        }

        public static function getAddInfoE1( $status )
        {
            $retorno = array( );
            if( $status == -1 )
                return "Inoperative";
            if( $status == 0 )
                return "Free";
            else if( $status == 6 )
                return "Locked";
            else
            {
                $mascaras = array( Populator::zeroFill( decbin( 1 ), 8 ), Populator::zeroFill( decbin( 2 ), 8 ), Populator::zeroFill( decbin( 4 ), 8 ), Populator::zeroFill( decbin( 16 ), 8 ), Populator::zeroFill( decbin( 32 ), 8 ), Populator::zeroFill( decbin( 64 ), 8 ), Populator::zeroFill( decbin( 128 ), 8 ) );
                $status = decbin( $status );
                $status = Populator::zeroFill( $status, 8 );

                if( ($mascaras[2] & $status) == $mascaras[2] )
                    $retorno[] = "Incoming";

                if( ($mascaras[1] & $status) == $mascaras[1] )
                    $retorno[] = "Outgoing";

                if( ($mascaras[3] & $status) == $mascaras[3] )
                    $retorno[] = "Locked for Outgoing";

                if( ($mascaras[4] & $status) == $mascaras[4] )
                    $retorno[] = "Fail";

                if( ($mascaras[5] & $status) == $mascaras[5] )
                    $retorno[] = "Locked for Incoming";

                if( ($mascaras[6] & $status) == $mascaras[6] )
                    $retorno[] = "Remote Lock";
            }
            return implode( ",", $retorno );
        }

        public static function zeroFill( $var, $size )
        {
            $aux = "";
            if( strlen( $var ) >= $size )
                return $var;
            for( $x = strlen( $var ); $x < $size; $x++ )
                $aux .= '0';
            return $aux . $var;
        }

        public static function linkCount( $serial )
        {
            $links_count = 0;
            $query_link_count = sprintf( "QUERY k3l.Config.Device.%s.LinkCount", $serial );
            Querist::ask( $query_link_count, $links_count );
            return $links_count;
        }

        public static function getDeviceModel( $type, $serial )
        {
            Global $boardModel;

            $model = '';
            $link_count = '';
            $voipCount = "";

            if( $type == 17 )
                return 'VoIP';

            $query_model = sprintf( "QUERY k3l.Config.Device.%s.DeviceModel", $serial );
            Querist::ask( $query_model, $model );
            $link_count = Populator::linkCount( $serial );

            $query_voip = sprintf( 'QUERY k3l.Config.Device.%s.VoIPChannelCount', $serial );
            Querist::ask( $query_voip, $voipCount );

            // echo "@".$type."@".$model."@".$link_count."@".$voipCount."<BR>";

            return $boardModel[$type][$model][$link_count][$voipCount];
        }

        public static function getEBSDeviceModel( $model )
        {
            switch ($model)
            {
                case '0':
                    return "CTI";
                case '1':
                    return "SPX";
                default:
                    return "-";
            }
        }

        public static function checkValue( $value )
        {
            if( $value == "" || $value == " " || $value == null )
                return "N/A";
            else
                return $value;
        }

    }
?>