<?php
    /**
     * Application base class
     */
    final class KSystem
    {
        /**
         * Current language
         */
        private $language = null;
        /**
         * Current version
         */
        private $version;
        /**
         * Self reference
         */
        static private $self = null;
        /**
         * WARNING: Singleton implementation!
         */
        private function __construct( )
        {
            @session_start( );
            date_default_timezone_set( 'Brazil/East' );

            if( array_key_exists( 'language', $_GET ) )
            {
                $this->setLanguage( $_GET['language'] );
            }
            elseif( array_key_exists( 'language', $_SESSION ) )
            {
                $this->setLanguage( $_SESSION['language'] );
            }
            else
            {
                $this->detectLanguage( );
            }
        }

        static public function getInstance( )
        {
            if( is_null( self::$self ) )
            {
                self::$self = new KSystem( );
            }
            return self::$self;
        }

        /**
         * Translate word
         */
        public function translate( $word )
        {
            //TODO: Change the dictionary to be loaded by the setLanguage
            // function
            //      as an protected attribute of this class
            global $dicionario;

            if( isset( $dicionario[$this->getLanguage( )][$word] ) )
            {
                return $dicionario[$this->getLanguage( )][$word];
            }
            else
            {
                return $word;
            }
        }

        /**
         * Detect and set language from browser headers or default to pt-br
         */
        public function detectLanguage( )
        {
            $this->setLanguage( 'pt-br' );
        }

        public function setLanguage( $lang )
        {
            $this->language = $lang;
            $_SESSION['language'] = $lang;
        }

        public function getLanguage( )
        {
            return $this->language;
        }

        /**
         * Get the root configuration directory
         */
        public function getConfigDir( )
        {
            if( PHP_OS == 'WINNT' )
            {
                $winVersion = PHP_WINDOWS_VERSION_MAJOR . "." . PHP_WINDOWS_VERSION_MINOR;
                exec( KROOT . '\\..\\..\\Tools\\kwebhelper configdir', $sysdir );
                if( $winVersion >= 6.1 )
                {
                    $sysdata = explode( "\\", $sysdir[0] );

                    if( sizeof( $sysdata ) <= 3 )
                    {
                        return "{$sysdata[0]}\\ProgramData\\{$sysdata[1]}\\{$sysdata[2]}";
                    }
                    else
                    {
                        return $sysdir[0];
                    }
                }
                return null;
            }
            else
            {
                return "/etc/khomp/config";
            }
        }

        /**
         * Get the installation directory
         */
        public function getInstallDir( )
        {
            if( PHP_OS == 'WINNT' )
            {
                $tmpDir = explode( "\\", $this->getConfigDir( ) );
                array_pop( $tmpDir );
                return implode( "\\", $tmpDir );
            }
            else
            {
                return '/etc/khomp';
            }
        }

        /**
         * Get the package version
         */
        public function getPackageVersion( )
        {
            $install = $this->getInstallDir( );
            if( !is_null( $install ) )
            {
                if( PHP_OS == 'WINNT' )
                {
                    $file = "{$install}\\3.1.0\\k3l.dat";
                }
                else
                {
                    $file = "{$install}/3.1.0/k3l.dat";
                }
                if( file_exists( $file ) )
                {
                    $lines = file( $file );
                    foreach( $lines as $line )
                    {
                        $line = trim( $line );
                        if( strpos( $line, 'LastBuildInstalled' ) === 0 )
                        {
                            $start = strpos( $line, '=' ) + 1;
                            return substr( $line, $start, strlen( $line ) - $start );
                        }
                    }
                }
            }
            return $this->translate( 'InfoNotAvailable' );
        }

        /**
         * Redirect to login page, if the user is not logged in
         */
        public function redirectIfNotLogged( )
        {
            if( !array_key_exists( 'login', $_SESSION ) )
            {
                header( 'Location: /?error=Kerr3' );
                exit ;
            }
        }

        /**
         * Check access permission
         *
         * @param location string System location
         * @param type string|array Required permission type or permission types
         * @return boolean true if has permission (if type is string) or any
         * permission
         *    (if type is array)
         */
        public function hasPermission( $location, $type = 'full' )
        {
            if( array_key_exists( 'loginAccess', $_SESSION ) && array_key_exists( $location, $_SESSION['loginAccess'] ) )
            {
                if( is_array( $type ) )
                {
                    return in_array( $_SESSION['loginAccess'][$location], $type );
                }
                else
                {
                    return $_SESSION['loginAccess'][$location] == $type;
                }
            }
            return false;
        }

    }
