#!/bin/bash -l

###############################################################################

REVISION_NUMB="$Rev: 7091 $"
REVISION_DATE="$LastChangedDate: 2012-09-13 18:12:48 -0300 (Thu, 13 Sep 2012) $"

export PATH=${PATH}
#DEFAULT_LINE_STRING="-----------------------------------------------------------------------------"
DEFAULT_LINE_STRING=""
CHARSET_TRANSLATION="iconv -f 'UTF-8'"

if ! which 'iconv' &> /dev/null
then
    CHARSET_TRANSLATION="cat"
fi

PROG_NAME="$0"

FIX_TMP_DIR="/tmp/.kwebportal-log-tmpdir.txt"
LOG_DIR="/var/log/khomp"
LOG_SOFTPBX="${LOG_DIR}/softpbx.log"
LOGFILE="${LOG_DIR}/diagnostic.log"
DIALPLAN="${LOG_DIR}/dialplan.log"

SOFTPBX=0

############################## CODES ###########################################
SUCCESS=0 # 
ERROR=1   # Generic error
EROOT=2   # user != root
ENFIND=3  # Programs not found
EK3L=4    # k3lserver is not actived
ECHAN=5   # chan_khomp is not loaded
EMOD=6    # mod_khomp is not loaded
ESOFT=7   # Softpbx is not actived
###############################################################################
main_tabulate ()
{
    local count="$1"
    local tabs=""

    for ((i=0;i<count;i++))
    do
        tabs="${tabs}  "
    done

    while read LINE
    do
        echo "${tabs}${LINE}"
    done
}

msg_prefix ()
{
    echo "[$(date '+%d/%m %H:%M:%S')]"
}

msg_nonl ()
{
    (echo -n "$@" | ${CHARSET_TRANSLATION}) >&2

    (echo "$@" | ${CHARSET_TRANSLATION}) >> "${LOGFILE}"
}

msg ()
{
    (echo "$@" | ${CHARSET_TRANSLATION}) >&2

    (echo "$@" | ${CHARSET_TRANSLATION}) >> "${LOGFILE}"
}

###############################################################################
asterisk_remote ()
{
#   asterisk -r -q -m -x "$@"
    asterisk -r -x "$@"
}

freeswitch_remote ()
{
    ${FREESWITCH_BIN_PATH}/fs_cli -x "$@"
}

softpbx_remote()
{
    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_remote "$@"
    elif [ ${SOFTPBX} == 2 ]
    then
        freeswitch_remote "$@"
    fi
}

###############################################################################
asterisk_log_commands()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "asterisk -V: " >> "${LOG_SOFTPBX}"
    asterisk -V >> "${LOG_SOFTPBX}"
    
    msg -n "."
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "khomp revision: " >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp revision" | grep '^<K>' >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "khomp summary:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp summary" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp links errors show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp links errors show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp links show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp links show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp channels show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp channels show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp channels statistics show:" >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp channels statistics show" | grep '^<K>' | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
}
        
freeswitch_log_commands()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "freeswitch -version: " >> "${LOG_SOFTPBX}"
    ${FREESWITCH_BIN_PATH}/freeswitch -version >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo -n "khomp revision: " >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp revision" >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${LOG_SOFTPBX}"
    echo "khomp summary:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp summary" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show links errors:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show links errors" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show links:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show links" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show channels:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show channels" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."


    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "khomp show statistics:" >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp show statistics" | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
}

generic_log_commands()
{
    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_log_commands $@
    else
        freeswitch_log_commands $@
    fi
}

asterisk_log_start()
{
    #ROTATE
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Rotating Asterisk log files" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    asterisk_remote "logger rotate" | main_tabulate 2 >> "${LOG_SOFTPBX}"

    #LER NO KLOG.CFG SE É PARA ATIVAR DEBUGGING OU ALL
    #level=`cat /etc/khomp/config/klog.cfg | grep -A 1 "\\[Channel\\]" | grep Value | cut -d= -f2`
    #HABILITA OS LOGS DA API/CHANNEL
    #echo "asterisk_remote \"khomp log disk just $level\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    #asterisk_remote "khomp log disk just $level" | grep '^<K>' | main_tabulate 2 >> "${LOG_SOFTPBX}"
    echo "asterisk_remote \"khomp log disk diagnostic\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    asterisk_remote "khomp log disk diagnostic" | grep '^<K>' | main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    # HABILITA LOGS DO ASTERISK (TODO: CHECAR VERSAO)
    echo "asterisk_remote \"set verbose 99\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    asterisk_remote "set verbose 99" | main_tabulate 2 >> "${LOG_SOFTPBX}"
    asterisk_remote "core set verbose 99" | main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    echo "asterisk_remote \"set debug 99\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    asterisk_remote "set debug 99" | main_tabulate 2 >> "${LOG_SOFTPBX}"
    asterisk_remote "core set debug 99" | main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    asterisk_remote "dialplan show" > "${DIALPLAN}"

    msg -n "."

#    local basekhompconfdir="${basedir}/khomp/config"
#    local baseastconfdir="${basedir}/asterisk/config"
#
#    # COPIAR CONFIGURACOES
#    cp -rL /etc/khomp/config/ "${basekhompconfdir}"
#
#    if [ -e '/etc/asterisk/asterisk.conf' ]
#    then
#        cp -rL '/etc/asterisk' "${baseastconfdir}"
#    else
#        local astconfdir=$(dirname `locate extconfig.conf | grep -v 'log' | tail -1`)
#
#        if [ -e "${astconfdir}" ]
#        then
#            cp -rL "${astconfdir}" "${baseastconfdir}"
#        fi
#    fi
#
#    msg "."
}

freeswitch_log_start()
{
    #ROTATE
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Rotating FreeSWITCH log files" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    freeswitch_remote "fsctl send_sighup" | main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    #LER NO KLOG.CFG SE É PARA ATIVAR DEBUGGING OU ALL
    #level=`cat /etc/khomp/config/klog.cfg | grep -A 1 "\\[Channel\\]" | grep Value | cut -d= -f2`
    #HABILITA OS LOGS DA API/ENDPOINT
    #echo "freeswitch_remote \"khomp log disk just $level\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    #freeswitch_remote "khomp log disk just $level" |  main_tabulate 2 >> "${LOG_SOFTPBX}"
    echo "freeswitch_remote \"khomp log disk diagnostic\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    freeswitch_remote "khomp log disk diagnostic" |  main_tabulate 2 >> "${LOG_SOFTPBX}"

    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    # HABILITA LOGS DO FREESWITCH
    echo "freeswitch_remote \"fsctl loglevel debug\"" | main_tabulate 1 >> "${LOG_SOFTPBX}"
    freeswitch_remote "fsctl loglevel debug" | main_tabulate 2 >> "${LOG_SOFTPBX}"
    
    msg -n "."

    echo >> "${LOG_SOFTPBX}"

    cp "${FREESWITCH_INSTALL_PATH}/log/freeswitch.xml.fsxml" "${DIALPLAN}"

    msg -n "."

#    echo >> "${LOG_SOFTPBX}"

#    local basekhompconfdir="${basedir}/khomp/config"
#    local basefsconfdir="${basedir}/freeswitch/config"
#
#    # COPIAR CONFIGURACOES
#    cp -rL /etc/khomp/config/ "${basekhompconfdir}"
#
#    if [ -e "${FREESWITCH_INSTALL_PATH}/conf/freeswitch.xml" ]
#    then
#        cp -rL "${FREESWITCH_INSTALL_PATH}/conf" "${basefsconfdir}"
#    else
#        local fsconfdir=$(dirname `locate freeswitch.serial | grep -v 'log' | tail -1`)
#
#        if [ -e "${fsconfdir}" ]
#        then
#            cp -rL "${fsconfdir}" "${basefsconfdir}"
#        fi
#    fi
#
#    msg "."
}

generic_log_start()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Enabling SoftPBX logs:" >> "${LOG_SOFTPBX}"

    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_log_start $@
    else
        freeswitch_log_start $@
    fi
}

k3l_log_start()
{
    echo "Enabling K3L FullLog" >> "${LOG_SOFTPBX}"
    cat /etc/khomp/config/klog.cfg | sed 's/FullLog=false/FullLog=true/g' > /tmp/.start.klog.cfg.tmp
    mv /tmp/.start.klog.cfg.tmp /etc/khomp/config/klog.cfg

    /usr/sbin/kwebhelper reload
    
    echo "OK" | main_tabulate 1 >> "${LOG_SOFTPBX}"
}

main_log_start ()
{

    softpbx_remote "khomp log console just standard" > /dev/null

    # ACHA A ULTIMA PASTA DE LOGS CRIADA E VERSÃO DA API
    local k3l_summary=`softpbx_remote 'khomp summary' | grep 'K3L API'`

    local k3l_version=`expr "${k3l_summary}" : '.\+K3L API \([0-9.]\+\).\+' | cut -d. -f1-2`

    msg # empty line.

    msg -n "$(msg_prefix) Enabling logs (version ${k3l_version}), waiting..."

    # INFORMACOES DO SISTEMA, VERSOES, FERRAMENTAS

    echo "${PROG_NAME} (${REVISION_NUMB}/${REVISION_DATE})" > "${LOG_SOFTPBX}"
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "################################################################################" >> "${LOG_SOFTPBX}"    
    echo "[$(date '+%d/%m %H:%M:%S')] START" >> "${LOG_SOFTPBX}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "uname -a: " >> "${LOG_SOFTPBX}"
    uname -a | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "gcc --version: " >> "${LOG_SOFTPBX}"
    gcc --version | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."
    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "ldd --version: " >> "${LOG_SOFTPBX}"
    ldd --version | main_tabulate 1 >> "${LOG_SOFTPBX}"

    msg -n "."

    generic_log_commands
    
    msg -n "."
    
    generic_log_start

    msg -n "."

    k3l_log_start
   
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "################################################################################" >> "${LOG_SOFTPBX}"    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"

    msg "."

    msg "$(msg_prefix) Logs enabled."
    msg # empty line.
}

###############################################################################
asterisk_log_stop()
{
    msg -n "."
    # DLITA LOG DE COMANDOS/EVENTOS DA API E DO CHANNEL DA KHOMP
    asterisk_remote "khomp log disk just standard" > /dev/null

    msg -n "."

    # DESABILITA LOGS DO ASTERISK
    asterisk_remote "set verbose 0" > /dev/null
    asterisk_remote "core set verbose 0" > /dev/null
    asterisk_remote "set debug 0" > /dev/null
    asterisk_remote "core set debug 0" > /dev/null
    
    msg -n "."
}

freeswitch_log_stop()
{
    msg -n "."

    # DESABILITA LOG DE COMANDOS/EVENTOS DA API E DO ENDPOINT DA KHOMP
    freeswitch_remote "khomp log disk just standard" > /dev/null

    msg -n "."

    # DESABILITA LOGS DO FREESWITCH
    freeswitch_remote "fsctl loglevel notice" > /dev/null

    msg -n "."
}

generic_log_stop()
{
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
    echo "Disabling SoftPBX logs:" >> "${LOG_SOFTPBX}"

    if [ ${SOFTPBX} == 1 ]
    then
        asterisk_log_stop $@
    else
        freeswitch_log_stop $@
    fi
    
    echo "OK" | main_tabulate 1 >> "${LOG_SOFTPBX}"
}

k3l_log_stop()
{
    echo "Disabling K3L FullLog" >> "${LOG_SOFTPBX}"
    cat /etc/khomp/config/klog.cfg | sed 's/FullLog=true/FullLog=false/g' > /tmp/.stop.klog.cfg.tmp
    mv /tmp/.stop.klog.cfg.tmp /etc/khomp/config/klog.cfg

    /usr/sbin/kwebhelper reload

    echo "OK" | main_tabulate 1 >> "${LOG_SOFTPBX}"
}

main_log_stop ()
{
    msg # empty line.
    
    echo "[$(date '+%d/%m %H:%M:%S')] STOP" >> "${LOG_SOFTPBX}"

    msg -n "$(msg_prefix) Disabling logs..."
    
    generic_log_stop

    msg -n "."

    k3l_log_stop

    msg "."

    msg "$(msg_prefix) Logs disabled."
    msg # empty line.
    
    echo "################################################################################" >> "${LOG_SOFTPBX}"    
    echo "${DEFAULT_LINE_STRING}" >> "${LOG_SOFTPBX}"
}

asterisk_status()
{
    local log_to_disk=`asterisk_remote "khomp log status"| grep "Enabled log-on-disk"`

    if echo ${log_to_disk} | grep -q "diagnostic"
    then
        return 0
    fi
    
    return 1
}

freeswitch_status()
{
    local log_to_disk=`freeswitch_remote "khomp log status"| grep "Enabled log-on-disk"`

    if echo ${log_to_disk} | grep -q "diagnostic"
    then
        return 0
    fi
    
    return 1
}

generic_status()
{
    if [ ${SOFTPBX} == 1 ]
    then
        return $(asterisk_status $@)
    else
        return $(freeswitch_status $@)
    fi
}

k3l_status()
{
    if cat /etc/khomp/config/klog.cfg | grep -q "FullLog[ ]*=[ ]*true"
    then
        return 0
    fi

    return 1
}

main_status()
{
    return $(generic_status && k3l_status)
}

###############################################################################
main_help()
{
    echo
    echo "$0: Enable logs tool."
    echo
    echo "Usage: $0 [options]"
    echo
    echo "Options:"
    echo
    echo "  --start      Start logs."
    echo "  --stop       Stop logs."
    echo "  --help | -h  This help screen."
    echo
}
###############################################################################

if [ "$1" = "help" ] || [ "$1" = "--help" ] || [ "$1" = "-h" ]
then
    main_help
    exit ${SUCCESS}
fi

# VERIFICA SE EH SUPER-USUARIO
if [ `whoami` != root ]
then
    msg "${PROG_NAME}: ERROR: Only root can do that."
    exit ${EROOT}
fi

# VERIFICA SE TEM DEPENDENCIAS
if ! which 'find' 'expr' &> /dev/null
then
    msg "${PROG_NAME}: ERROR: Programs 'find' and 'expr' not found."
    msg
    exit ${ENFIND}
fi

if ! pidof k3lserver &>/dev/null
then
    msg "${PROG_NAME}: ERROR: k3lserver is not running."
    exit ${EK3L}
fi

if pidof asterisk &>/dev/null
then
    SOFTPBX=1

    if asterisk_remote "khomp summary" | grep -q "No such command"
    then
        msg "${PROG_NAME}: ERROR: \"chan_khomp\" is not loaded in the Asterisk"
        exit ${ECHAN}
    fi

elif pidof freeswitch &>/dev/null
then
    SOFTPBX=2
    if [ "${FREESWITCH_INSTALL_PATH}w" == "w" ]
    then
        FREESWITCH_INSTALL_PATH="/usr/local/freeswitch"
    fi

    if ! which 'freeswitch' &> /dev/null
    then
        FREESWITCH_BIN_PATH="${FREESWITCH_INSTALL_PATH}/bin"
    else
        FREESWITCH=`which freeswitch`
        FREESWITCH_BIN_PATH=`dirname ${FREESWITCH}`
        FREESWITCH_INSTALL_PATH=`echo ${FREESWITCH_BIN_PATH} | sed 's/\/bin[/]\?$//'`
    fi

    if [ ! -x ${FREESWITCH_BIN_PATH}/fs_cli ]
    then
        FS_PID=`pidof freeswitch`
        FREESWITCH=`ls -la /proc/${FS_PID}/exe | grep "\->" | sed 's/.* -> \(.*\)$/\1/'`
        FREESWITCH_BIN_PATH=`dirname ${FREESWITCH}`
        FREESWITCH_INSTALL_PATH=`echo ${FREESWITCH_BIN_PATH} | sed 's/\/bin[/]\?$//'`
    fi
    
    if freeswitch_remote "khomp summary" | grep -qi "not found"
    then
        msg "${PROG_NAME}: ERROR: \"mod_khomp\" is not loaded in the FreeSWITCH"
        exit ${EMOD}
    fi
else
    msg "${PROG_NAME}: ERROR: SoftPBX is not running"
   exit ${ESOFT}
fi

if [ $# == 0 ] || [ "$1" == "start" ] || [ "$1" == "--start" ]
then
    main_log_start

elif [ "$1" == "stop" ] || [ "$1" == "--stop" ]
then
    main_log_stop

elif [ "$1" == "status" ] || [ "$1" == "--status" ]
then
    if (main_status)
    then
        (echo "ACTIVE" | ${CHARSET_TRANSLATION}) 
    else
        (echo "INACTIVE" | ${CHARSET_TRANSLATION}) 
    fi
    
else
    main_help
    exit ${ERROR}
fi

exit ${SUCCESS}

