<?php
require_once KCONFIG . '/classes/classKGatewayUtil.php';
require_once KCONFIG . '/classes/classKYaml.php';

$AjaxFunctions['showCDROptions'] = 'showCDROptions';
$AjaxFunctions['saveCDROptions'] = 'saveCDROptions';

function CDROption( $app, $name, $format )
{
    $trvar = "gateway-cdr-{$name}";
    $cdrval = is_null( $format ) ? $name : "{$name}:{$format}";
    return "<li cdr-value='{$cdrval}' class='gateway-cdr-option ui-dragable'>{$name} <img src='/imagens/tooltip.png' class='gateway-tooltip' alt='tooltip' khelp='{$app->translate( $trvar )}' /></li>";
}
function CDRRotateOption( $app, $name, $current )
{
    $s = $name == $current ? ' selected=\'selected\'' : '';
    $n = 'gateway-cdr-' . str_replace( ' ', '-', $name );
    return "<option value='{$name}'{$s}>{$app->translate( $n )}</option>";
}

function showCDROptions( $request )
{
    $app = KSystem::getInstance();
    // available options to gateway-cdr-select
    $options = array(
        'callid', 'leg_a', 'leg_b', 'orig_addr', 'dest_addr', 'leg_b_orig_addr',
        'leg_b_dest_addr', 'start_stamp', 'answer_stamp', 'end_stamp',
        'duration', 'billsec', 'hangup_origin', 'hangup_cause', 'dest_nap',
        'route_name'
    );
    // default option format
    $options_fmt = array(
        'start_stamp'=>'%d-%m-%y %H:%M:%S',
        'answer_stamp'=>'%d-%m-%y %H:%M:%S',
        'end_stamp'=>'%d-%m-%y %H:%M:%S'
    );
    // rotate options
    $rotate = array( 'every minute', 'hourly', 'daily' );
    // samples (array of array, the second array has the sample name on the
    // first position, the second has the sample format and from the third
    // position to the end of array are the sample outputs
    $samples = array(
        array(
            'CSV',
            '"${callid}", "${leg_a}", "${leg_b}", "${orig_addr}", "${dest_addr}", "${leg_b_orig_addr}", "${leg_b_dest_addr}", "${start_stamp:%d/%m/%y %H:%M:%S}", "${answer_stamp:%d/%m/%y %H:%M:%S}", "${end_stamp:%d/%m/%y %H:%M:%S}", "${duration}", "${billsec}", "${hangup_origin}", "${hangup_cause}", "${dest_nap}", "${route_name}"',
            '"0526870051344016125586952140", "D52687 C5", "D52687 C6", "551151823411", "554837222900", "551151823411", "554837222900", "03/08/12 14:46:17", "03/08/12 14:46:27", "03/08/12 14:48:19", "122", "112", "leg a", "0", "nap a", "default"',
            '"0526870161344016811385295214", "D52687 C16", "D52687 C4", "554837222900", "551151823411", "554837222900", "551151823411", "03/08/12 15:00:07", "03/08/12 15:00:25", "03/08/12 15:32:25", "1938", "1920", "leg a", "0", "nap a", "default"',
            '"0526870011344020110345605789", "D52687 C1", "D19432 C9", "551151823411", "541159176025", "551151823411", "541159176025", "03/08/12 21:12:55", "03/08/12 21:13:16", "03/08/12 21:22:00", "545", "524", "leg b", "4", "nap c", "argentina"'
        ),
        array(
            'SQL',
            'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("${callid}", "${orig_addr}", "${dest_addr}", "${start_stamp:%Y-%m-%d %H:%M:%S}", "${answer_stamp:%Y-%m-%d %H:%M:%S}", "${end_stamp:%Y-%m-%d %H:%M:%S}");',
            'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("0526870051344016125586952140", "551151823411", "554837222900", "2012-08-03 14:46:17", "2012-08-03 14:46:27", "2012-08-03 14:48:19");',
            'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("0526870161344016811385295214", "554837222900", "551151823411", "2012-08-03 15:00:07", "2012-08-03 15:00:25", "2012-08-03 15:32:25");',
            'INSERT INTO CallLog (`callid`, `orig_addr`, `dest_addr`, `start`, `answer`, `end`) VALUES ("0526870011344020110345605789", "551151823411", "541159176025", "2012-08-03 21:12:55", "2012-08-03 21:13:16", "2012-08-03 21:22:00");'
        ),
        array(
            $app->translate( 'gateway-cdr-sample-custom' ),
            '${callid}: (${leg_a}) [${orig_addr} -> ${dest_addr}] <---> ${route_name} <---> (${leg_b}) [${leg_b_orig_addr} -> ${leg_b_dest_addr}] -- start=${start_stamp:%c}, answer=${answer_stamp:%c}, end=${end_stamp:%c}, total=${duration}, bill=${billsec}',
            '0526870051344016125586952140: (D52687 C5) [551151823411 -> 554837222900] <---> default <---> (D52687 C6) [551151823411 -> 554837222900] -- start=Fri Aug 03 14:46:17 2012, answer=Fri Aug 03 14:46:27 2012, end=Fri Aug 03 14:48:19 2012, total=122, bill=112',
            '0526870161344016811385295214: (D52687 C16) [554837222900 -> 551151823411] <---> default <---> (D52687 C4) [554837222900 -> 551151823411] -- start=Fri Aug 03 15:00:07 2012, answer=Fri Aug 03 15:00:25 2012, end=Fri Aug 03 15:32:25 2012, total=1938, bill=1920',
            '0526870011344020110345605789: (D52687 C1) [551151823411 -> 541159176025] <---> argentina <---> (D19432 C9) [551151823411 -> 541159176025] -- start=Fri Aug 03 21:12:55 2012, answer=Fri Aug 03 21:13:16 2012, end=Fri Aug 03 21:22:00 2012, total=545, bill=524'
        )
    );

    $kyaml = new KYaml( GatewayUtil::getGatewayConfigPath() );
    $yaml = $kyaml->getYaml();

    $buf = '<div id="gateway-cdr-config"><fieldset name="gateway-cdr" id="gateway-cdr"><legend>CDR</legend>';
    $buf .= '<table name="gateway-cdr-table" id="gateway-cdr-table">';
    $buf .= "<tr><td class='gateway-cdr-td-align-left'><label for='gateway-cdr-rotate'>{$app->translate( 'gateway-cdr-rotate-title' )}</label>: <select name='gateway-cdr-rotate' id='gateway-cdr-rotate'>";
    foreach( $rotate as &$opt )
    {
        $buf .= CDRRotateOption( $app, $opt, $yaml['CDR']['Rotate'] );
    }
    $buf .= '</select></td>'; // end of rotation select
    $buf .= "<td width='150px' rowspan='3'><p>{$app->translate( 'gateway-cdr-options-title' )} <img src='/imagens/tooltip.png' class='gateway-tooltip' alt='tooltip' khelp='{$app->translate( 'gateway-cdr-options-help' )}' /></p><ul id='gateway-cdr-select' name='gateway-cdr-select'>";
    foreach( $options as &$opt )
    {
        $fmt = array_key_exists( $opt, $options_fmt ) ? $options_fmt[$opt] : null;
        $buf .= CDROption( $app, $opt, $fmt );
    }
    $buf .= '</ul></td></tr>'; // end of pattern editor variables
    $buf .= "<tr><td class='gateway-cdr-td-align-left'><label for='gateway-cdr-pattern'>{$app->translate( 'gateway-cdr-pattern-title' )}</label>:</td></tr><tr><td><div><textarea name='gateway-cdr-pattern' id='gateway-cdr-pattern'>{$yaml['CDR']['Pattern']}</textarea></div></td></tr>"; // end of pattern editor
    $buf .= "<tr><td colspan='2'><div id='gateway-cdr-help' name='gateway-cdr-help'><span class='gateway-cdr-help-title'>+ {$app->translate( 'gateway-cdr-fmt-title' )} <img src='/imagens/tooltip.png' alt='tooltip' /></span><table class='help'>{$app->translate( 'gateway-cdr-fmt' )}</table><br /><span class='gateway-cdr-help-title'>+ {$app->translate( 'gateway-cdr-sample-title' )} <img src='/imagens/tooltip.png' alt='tooltip' /></span><div class='help'>";
    foreach( $samples as &$sample )
    {
        $buf .= "<p>{$app->translate( 'gateway-cdr-sample-fmt' )} <span class='bld'>{$sample[0]}</span>: <span class='gateway-cdr-help-sample'>{$sample[1]}</span> <span class='gateway-cdr-help-sample-copy'>{$app->translate( 'gateway-cdr-sample-copy' )}</span><br />{$app->translate( 'gateway-cdr-sample-result' )}:<br />";
        for( $r = 2; $r < count( $sample ); $r++ )
        {
            $buf .= "<span class='gateway-cdr-help-sample'>{$sample[$r]}</span><br />";
        }
        $buf .= '</p>';
    }
    $buf .= '</div></div></td></tr>'; // end of help sections
    $buf .= '</table></fieldset>'; // end of fieldset
    $buf .= "<br /><input type='button' id='gateway-cdr-save' name='gateway-cdr-save' value='{$app->translate( 'save' )}' />";
    return $buf;
}

function saveCDROptions( $request )
{
    $app = KSystem::getInstance();

    if( ! array_key_exists( 'pattern', $request ) || ! array_key_exists( 'rotate', $request ) )
    {
        header( 'Status: 400 Bad Request' );
        return $app->translate( 'gateway-cdr-save-error-notnull' );
    }
    elseif( strpos( $request['pattern'], "\n" ) !== false )
    {
        header( 'Status: 400 Bad Request' );
        return $app->translate( 'gateway-cdr-save-error-nolinebreak' );
    }

    $kyaml = new KYaml( GatewayUtil::getGatewayConfigPath() );
    $yaml = $kyaml->getYaml();

    $yaml['CDR']['Pattern'] =  $request['pattern'];
    $yaml['CDR']['Rotate'] =  $request['rotate'];


    $kyaml->setYaml( $yaml );
    $kyaml->saveYaml();

    return $app->translate( 'gateway-cdr-saved' );
}

