<?php
    // error_reporting(E_ALL);
    require_once KROOT . '/util.php';
    require_once KCONFIG . '/classes/classKSistema.php';
    require_once KMONITOR . '/languages/dictionary.php';
    require_once KMONITOR . '/querist.php';

    header( 'Content-Type: text/html; charset=UTF-8' );

    class MonitorService
    {
        function __construct( )
        {
            Global $_SESSION, $sistema;
            $service = array( );

            if( PHP_OS == "WINNT" )
            {
                $service['Khomp Integrated Boot System'] = MonitorService::checkService( "kibs" );
                $service['Khomp Log Server'] = MonitorService::checkService( "klogserver" );
                $service['Khomp Media Server'] = MonitorService::checkService( "kmpserver" );
                $service['Khomp API Server'] = MonitorService::checkService( "K3LServer" );
                $service['Khomp Query Server'] = MonitorService::checkService( "KQueryServer" );
            }
            else
            {
                $service['Khomp Integrated Boot System'] = MonitorService::checkService( "kibs" );
                $service['Khomp Log Server'] = MonitorService::checkService( "klogserver" );

                if( is_file( "/usr/sbin/kmpserver" ) )
                    $service['Khomp Media Server'] = MonitorService::checkService( "kmpserver" );
                elseif( MonitorService::checkPID( "asterisk" ) )
                    $service['SoftPBX']['Asterisk'] = true;
                elseif( MonitorService::checkPID( "freeswitch" ) )
                    $service['SoftPBX']['FreeSWITCH'] = true;
                else
                    $service['SoftPBX'] = false;

                $service['Khomp API Server'] = MonitorService::checkService( "k3lserver" );
                $service['Khomp Query Server'] = MonitorService::checkService( "kqueryserver" );

                if( is_file( '/usr/sbin/kgateway' ) )
                    $service['Khomp Gateway'] = MonitorService::checkService( "kgateway" );
            }
            $op = -1;
            $operational = Querist::ask( "QUERY k3l.Connected", $op );

            if( $op > 0 )
                unset( $_SESSION['errorMessage'] );
            else
            {
                if( !$operational )
                {
                    unset( $_SESSION['DeviceList'] );
                    $_SESSION['errorMessage'] = $sistema->retTranslate( Querist::$last_error );
                }
                else
                    unset( $_SESSION['DeviceList'] );
            }
            if( !isset( $_SESSION['errorMessage'] ) )
                $_SESSION['errorMessage'] = MonitorService::checkMediaServerMessages( );

            $_SESSION['ServiceList'] = $service;
        }

        public static function checkEssentialServiceAvailable( )
        {
            if( PHP_OS == "WINNT" )
                return MonitorService::checkService( "k3lserver" ) && MonitorService::checkService( "kqueryserver" ) && MonitorService::checkService( "kmpserver" );
            else
            {
                if( is_file( "/usr/sbin/kmpserver" ) )
                    return MonitorService::checkService( "k3lserver" ) && MonitorService::checkService( "kqueryserver" ) && MonitorService::checkService( "kmpserver" );
                elseif( MonitorService::checkPID( "asterisk" ) )
                    return MonitorService::checkService( "k3lserver" ) && MonitorService::checkService( "kqueryserver" ) && MonitorService::checkPID( "asterisk" );
                elseif( MonitorService::checkPID( "freeswitch" ) )
                    return MonitorService::checkService( "k3lserver" ) && MonitorService::checkService( "kqueryserver" ) && MonitorService::checkPID( "freeswitch" );
            }
        }

        public static function isSoftPBXActive( )
        {
            if( PHP_OS == "WINNT" )
                return MonitorService::checkService( "kmpserver" );
            else
            {
                if( is_file( "/usr/sbin/kmpserver" ) )
                    return MonitorService::checkService( "kmpserver" );
                elseif( MonitorService::checkPID( "asterisk" ) )
                    return true;
                elseif( MonitorService::checkPID( "freeswitch" ) )
                    return true;
                return false;
            }
        }

        public static function getWarningMessage( )
        {
            switch (MonitorService::checkNonEssentialService())
            {
                case '0':
                    return 'LogServerInactive';
                    break;
                case '1':
                    return 'KibsInactive';
                    break;
                case '2':
                    return 'KibsAndLogServerAreInactive';
                    break;
                default:
                    return '';
                    break;
            }
        }

        public static function checkNonEssentialService( )
        {
            $log = MonitorService::checkService( "klogserver" );
            $kibs = MonitorService::checkService( "kibs" );

            if( !$log && !$kibs )
                return '2';
            else
            {
                if( !$log )
                    return '0';
                if( !$kibs )
                    return '1';
                return '3';
            }
        }

        public static function getProcessName( $service )
        {
            switch ($service)
            {
                case 'Khomp Media Server':
                    return "kmpserver";
                case 'Khomp Integrated Boot System':
                    return "kibs";
                case 'Khomp Log Server':
                    return "klogserver";
                case 'Khomp Query Server':
                    return "kqueryserver";
                case 'Khomp API Server':
                    return "k3lserver";
                case 'Asterisk':
                    return "asterisk";
                case 'FreeSWITCH':
                    return "freeswitch";
                case 'Khomp Gateway':
                    return "kgateway";
                default:
                    return "-";
            }
        }

        public static function checkMediaServerMessages( )
        {
            Global $sistema;
            if( MonitorService::checkService( 'k3lserver' ) )
            {
                if( !MonitorService::isSoftPBXActive( ) )
                {
                    if( PHP_OS != "WINNT" )
                    {
                        if( is_file( "/usr/sbin/kmpserver" ) )
                            return $sistema->retTranslate( 'KMPDown' );
                        else
                            return $sistema->retTranslate( 'softPBXDown' );
                    }
                    else
                        return $sistema->retTranslate( 'KMPDown' );
                }
            }
            else
                return $sistema->retTranslate( 'k3lDown' );
        }

        public static function checkPID( $service )
        {
            return (bool) exec( "ps cax | grep -i " . $service );
        }

        public static function checkService( $service_name )
        {
            if( PHP_OS == "WINNT" )
            {
                $scCheck = win32_query_service_status( $service_name );
                return (bool)$scCheck['ProcessId'];
            }
            else
                return (bool) exec( "ps cax | grep -i " . $service_name );
        }

        public static function showServices( )
        {
            Global $sistema, $_SESSION, $image_dir;
            $alertMessage = $sistema->retTranslate( MonitorService::getWarningMessage( ) );

            if( isset( $_SESSION['errorMessage'] ) && $_SESSION['errorMessage'] != '' )
            {
                $style1 = "display:block;";
                $style2 = "display:none;";
            }
            elseif( isset( $alertMessage ) && $alertMessage != '' )
            {
                $style2 = "display:block;";
                $style1 = "display:none;";
            }
            else
            {
                $style2 = "display:none;";
                $style1 = "display:none;";
            }

            $buffer = "";
            $buffer .= "<input type='hidden' value='services' id='current-monitoring' />";

            $buffer .= "<div id='divErrorMessage' class='errorMessage' style='$style1'>";
            $buffer .= "<img id='errorIco' src='$image_dir/imagens/error.png' />";
            $buffer .= "<div id='divErrorMessageLine'>" . $_SESSION['errorMessage'] . "</div>";
            $buffer .= "</div>";

            $buffer .= "<div id='divErrorMessage' class='errorMessage' style='$style2'>";
            $buffer .= "<img id='errorIco' src='$image_dir/imagens/error.png' />";
            $buffer .= "<div id='divAlertMessageLine'>" . $alertMessage . "</div>";
            $buffer .= "</div>";

            $buffer .= "<br>";
            $serviceList = $_SESSION['ServiceList'];

            $access = $_SESSION['loginAccess']['monitoring'];
            $buffer .= "<input type='hidden' id='lang' value='" . $sistema->getLanguage( ) . "' name='lang' />";
            $buffer .= "<input type='hidden' id='access' value='$access' name='access' />";

            $buffer .= "<table border=1 class='tabelaService'>";
            $buffer .= "<th class='service'>" . $sistema->retTranslate( "service" ) . "</th>";
            $buffer .= "<th class='status'>" . $sistema->retTranslate( "status" ) . "</th>";
            $buffer .= "<th class='process'>" . $sistema->retTranslate( "process" ) . "</th>";
            $buffer .= "<th class='options'>" . $sistema->retTranslate( "options" ) . "</th>";

            $i = 0;

            foreach( $serviceList as $key => $value )
            {
                if( $key == "SoftPBX" )
                {
                    if( isset( $serviceList["SoftPBX"]["Asterisk"] ) )
                    {
                        $key = "Asterisk";
                        $value = $serviceList["SoftPBX"]["Asterisk"];
                    }
                    if( isset( $serviceList["SoftPBX"]["FreeSWITCH"] ) )
                    {
                        $key = "FreeSWITCH";
                        $value = $serviceList["SoftPBX"]["FreeSWITCH"];
                    }
                }

                if( $i % 2 == 0 )
                    $style = "background: none repeat scroll 0 0 #FFFFFF;";
                else
                    $style = "background: none repeat scroll 0 0 #F5F5F5;";

                $buffer .= "<tr style='$style'>";
                $buffer .= "<td id='name-$key'>" . $key . "</td>";
                $buffer .= "<td>";
                if( $value == 0 )
                {
                    $buffer .= "<span id='status-$key'>";
                    $buffer .= "<b>" . $sistema->retTranslate( "inactive" ) . "</b>";
                    $buffer .= "</span>";
                }
                else
                {
                    $buffer .= "<span id='status-$key'>";
                    $buffer .= "<b>" . $sistema->retTranslate( "active" ) . "</b>";
                    $buffer .= "</span>";
                }
                $buffer .= "</td>";
                $buffer .= "<td id='process-$key'>" . MonitorService::getProcessName( $key ) . "</td>";
                $buffer .= "<td>";

                if( PHP_OS == "WINNT" )
                {
                    if( $_SESSION['loginAccess']['monitoring'] == 'full' )
                    {
                        if( $value == 0 )
                        {
                            $buffer .= "<span id='button-$key'>";
                            $buffer .= "<a onclick=\"startService('" . $key . "');updateDataServices(true);updateDataServices(true);\" ><img width='16' height='16' style='cursor:pointer;' src='$image_dir/imagens/play.png' border='0'></a>";
                            $buffer .= "</span>";
                        }
                        else
                        {
                            $buffer .= "<span id='button-$key'>";
                            $buffer .= "<a onclick=\"stopService('" . $key . "');updateDataServices(true);\" ><img width='16' height='16' style='cursor:pointer;' src='$image_dir/imagens/stop.png' border='0' ></a>";
                            $buffer .= "</span>";
                        }
                    }
                    else
                        $buffer .= "<span id='button-$key'>-</span>";
                }
                else
                {
                    $ret = shell_exec( 'whoami' );
                    if( $_SESSION['loginAccess']['monitoring'] == 'full' && ($key != "Asterisk" && $key != "FreeSWITCH" && $key != "SoftPBX") )
                    {
                        if( strcasecmp( $ret, 'root' ) )
                        {
                            if( $value == 0 )
                            {
                                $buffer .= "<span id='button-$key'>";
                                $buffer .= "<a onclick=\"startService('" . $key . "');updateDataServices(true);updateDataServices(true);\" ><img width='16' height='16' style='cursor:pointer;' src='$image_dir/imagens/play.png' border='0'></a>";
                                $buffer .= "</span>";
                            }
                            else
                            {
                                $buffer .= "<span id='button-$key'>";
                                $buffer .= "<a onclick=\"stopService('" . $key . "');updateDataServices(true);updateDataServices(true);\" ><img width='16' height='16' style='cursor:pointer;' src='$image_dir/imagens/stop.png' border='0' ></a>";
                                $buffer .= "</span>";
                            }
                        }
                        else
                            $buffer .= "<img src='$image_dir/imagens/noRoot.png' border='0' title='" . $sistema->retTranslate( 'noPermission' ) . "' >";
                    }
                    else
                        $buffer .= "<span id='button-$key'>-</span>";
                }
                $buffer .= "</td>";
                $i++;
            }
            $buffer .= "</tr>";
            $buffer .= "</tr>";
            $buffer .= "</table>";

            return $buffer;
        }

    }
